import org.apache.commons.lang3.StringUtils

insight = new InsightTool(logger: logger, insightAccessToken: insightAccessToken, apiClientFactory: apiClientFactory)

/**
 * Velocity tool to get Insight object data from the Insight Cloud REST API.
 *
 * @see https://documentation.riada.io/display/INSCLOUD/REST+API
 */
class InsightTool {
	def logger

	/* Insight authentication parameter. */
	def insightAccessToken

	def apiClientFactory

	def insightClient

	def slurper = new groovy.json.JsonSlurper()

	/**
	 * Returns the Insight objects for the passed Asset field value (as Groovy objects slurped from <code>https://documentation.riada.io/display/INSCLOUD/REST+API+-+Load<code>)
	 * or the exception object if something went wrong.
	 */
	def getInsightObjects(def customFieldValue) {
		try {
			def objectIds = customFieldValue.collect { StringUtils.substringAfterLast(it.originId, "_") }
			return fetchInsightObjects(objectIds)
		} catch (Throwable ex) {
			return ex
		}
	}

	def getAttributeValues(def attribute) {
		attribute.objectAttributeValues.collect { it.displayValue }
	}

	private def fetchInsightObjects(def objectIds) {
		if (!insightAccessToken) {
			throw new IllegalStateException('Failed to download Insight object data! (see Insight API guide: https://www.midori-global.com/products/better-excel-exporter-for-jira/cloud/documentation/integrations/insight): Insight Cloud REST API access token is not set')
		}
		if (!insightClient) {
			insightClient = apiClientFactory.createInsightClient(insightAccessToken)
		}
		return insightClient.getInsightObjects(objectIds)
	}
}
